﻿//	『昼夜逆転』工作室	http://jsdiy.web.fc2.com
//	FT232 7セグ1桁制御 ライブラリ
//	2011/11/10	v1.00	初版

using FTD2XX_NET;

namespace FT232Seg7Lib1
{
	/*	秋月電子FT232RLモジュールのピン配置
	   +----------+
	D0 |TXD    GND|
	D4 |DTR#   CB0|
	D2 |RTS#   CB1|
	   |VIO    VCC|
	D1 |RXD    RST|
	D7 |RI#    3V3|
	   |GND    CB3|
	D5 |DSR#   PU1|
	D6 |DCD#   PU2|
	D3 |CTS#   VCC|
	   |CB4    USB|(5V)
	   |CB2    SLD|(GND)
	   +----↓----+
	   USBコネクタ側
	*/
	/*	【7セグのセグメント図】
	+-A-+
	F   B
	+-G-+
	E   C
	+-D-+ DP
	*/
	
	/*	FT232RLモジュールと7セグのピンの対応
	セグメント：	A	B	C	D	E	F	G	DP
	モジュール：	D0	D1	D2	D3	D4	D5	D6	D7
	*/
	
	//7セグのセグメントパターン定義
	struct Segment
	{
		public const byte	//PGFEDCBA
			SegA	= 0x01,	//00000001
			SegB	= 0x02,	//00000010
			SegC	= 0x04,	//00000100
			SegD	= 0x08,	//00001000
			SegE	= 0x10,	//00010000
			SegF	= 0x20,	//00100000
			SegG	= 0x40,	//01000000
			SegDP	= 0x80,	//10000000
			SegNull	= 0x00;	//00000000

		public static readonly  byte[] SegAtoG = { SegA, SegB, SegC, SegD, SegE, SegF, SegG };
		
		private static readonly string[] DefNumber =
		{
		//	"0GFEDCBA"
			"00111111",	//0
			"00000110",	//1
			"01011011",	//2
			"01001111",	//3
			"01100110",	//4
			"01101101",	//5
			"01111101",	//6
			"00100111",	//7
			"01111111",	//8
			"01101111"	//9
		};
		
		private static readonly string[] DefUpperCase =
		{
		//	"0GFEDCBA"
			"01110111",	//A
			"00000000",	//(B)
			"00111001",	//C
			"00000000",	//(D)
			"01111001",	//E
			"01110001",	//F
			"00111101",	//G
			"01110110",	//H
			"00110000",	//I
			"00011110",	//J
			"01110101",	//K
			"00111000",	//L
			"00110111",	//M（'U'の逆さ）
			"00000000",	//(N)
			"00111111",	//O（'0'と同じ）
			"01110011",	//P
			"00111011",	//Q
			"00000000",	//(R)
			"01101101",	//S（'5'と同じ）
			"00110001",	//T（'「'の形）
			"00111110",	//U
			"01110010",	//V
			"01111110",	//W（'A'の逆さ）
			"01100100",	//X
			"01101110",	//Y
			"00011011"	//Z
		};
		
		private static readonly string[] DefLowerCase =
		{
		//	"0GFEDCBA
			"01011111",	//a
			"01111100",	//b
			"01011000",	//c
			"01011110",	//d
			"01111011",	//e
			"00000000",	//(f)
			"00000000",	//(g)
			"01110100",	//h
			"00010000",	//i
			"00001110",	//j
			"00000000",	//(k)
			"00110000",	//l（'I'と同じ）
			"00000000",	//(m)
			"01010100",	//n
			"01011100",	//o
			"00000000",	//(p)
			"01100111",	//q
			"01010000",	//r
			"00000000",	//(s)
			"01111000",	//t
			"00011100",	//u
			"00000000",	//(v)
			"00000000",	//(w)
			"00000000",	//(x)
			"00000000",	//(y)
			"00000000"	//(z)
		};
		
		private const string BITS_NULL =
			"00000000";
		
		public static byte GetNumberPattern(int num)
		{
			if (num < 0 || 9 < num) return SegNull;
			string bits = DefNumber[num];
			byte segPtn = BitStringToByte(bits);
			return segPtn;
		}
		
		public static byte GetAlphabetPattern(char cc)
		{
			string bits = BITS_NULL;
			
			if ('A' <= cc && cc <= 'Z')
			{
				int idx = cc - 'A';
				bits = DefUpperCase[idx];
				if (bits == BITS_NULL) bits = DefLowerCase[idx];
			}
			else if ('a' <= cc && cc <= 'z')
			{
				int idx = cc - 'a';
				bits = DefLowerCase[idx];
				if (bits == BITS_NULL) bits = DefUpperCase[idx];
			}
			
			byte segPtn = BitStringToByte(bits);
			return segPtn;
		}
		
		//8ビット2進数を表す文字列をbyte値に変換する
		public static byte BitStringToByte(string bits)
		{
			int n = 0x00;
			try
			{
				for (int i = 0; i < 8; i++)
					n = (n << 1) | (bits[i] == '0' ? 0 : 1);
			}
			catch { n = 0x00; }
			return (byte)n;
		}
	}
	
	class Seg7Ctrl
	{
		private readonly FTDI ftdev;
		private byte currentSegPtn;
		
		//FT232デバイスの通信エラーを知らせるフラグ
		public bool IsFtError { get; private set; }
		
		
		//コンストラクタ
		public Seg7Ctrl(FTDI ftdev)
		{
			this.ftdev = ftdev;
			currentSegPtn = Segment.SegNull;
			IsFtError = false;
		}
		
		public void PutNum(int num)
		{
			currentSegPtn = Segment.GetNumberPattern(num);
			SetPortBits(currentSegPtn);
		}
		
		public void PutChar(char cc)
		{
			currentSegPtn = Segment.GetAlphabetPattern(cc);
			SetPortBits(currentSegPtn);
		}
		
		public void PutSegPtn(byte segPtn)
		{
			currentSegPtn = segPtn;
			SetPortBits(currentSegPtn);
		}
		
		public void PutSegDP(bool dpOn)
		{
			if (dpOn)
				SetPortBits((byte)(currentSegPtn | Segment.SegDP));
			else
				SetPortBits(currentSegPtn);
		}
		
		//FT232 送受信バッファクリア
		public void BufferClear()
		{
			IsFtError = (ftdev.Purge(FTDI.FT_PURGE.FT_PURGE_RX | FTDI.FT_PURGE.FT_PURGE_TX) != FTDI.FT_STATUS.FT_OK);
		}
		
		//FT232 I/Oポートの各ビットにHi/Loをセットする（送信バッファに1byte書き込む）
		public void SetPortBits(byte bits)
		{
			uint writtenLength = 0;
			byte[] txBuf = { bits };
			IsFtError = (ftdev.Write(txBuf, 1, ref writtenLength) != FTDI.FT_STATUS.FT_OK);
		}
		
	}
}
