﻿//	『昼夜逆転』工作室	http://jsdiy.web.fc2.com
//	FT232 7セグ1桁制御 デモプログラム

using System;
using FTD2XX_NET;
using FT232Seg7Lib1;

namespace ft232r7seg1
{
	//終了コード
	enum ExitCode
	{
		NormalTermination = 0,
		FT232_OpenError = 10,
		FT232_InitializeError,
		FT232_Error,
		AnyReason = 255
	};
	
	class Program
	{
		private static FTDI ftdev = null;
		private static Seg7Ctrl segled = null;
		
		static void Main(string[] args)
		{
			ftdev = new FTDI();
			if (ftdev.OpenByIndex(0) != FTDI.FT_STATUS.FT_OK)
				AppClose("FT232 open error.", ExitCode.FT232_OpenError);
			if (ftdev.SetBitMode(0xFF, FTDI.FT_BIT_MODES.FT_BIT_MODE_ASYNC_BITBANG) != FTDI.FT_STATUS.FT_OK)
				AppClose("FT232 initialize error.", ExitCode.FT232_InitializeError);
			Console.WriteLine("FT232 open");
			Console.WriteLine();
			
			segled = new Seg7Ctrl(ftdev);
			
			Console.WriteLine("【7セグ点灯テスト 動作中】下記のキー入力を受け付けます。");
			Console.WriteLine();
			Console.WriteLine(
				"　0-9 a-z A-Z … 数字とアルファベット" + Environment.NewLine +
				"　. … 小数点の点灯と消灯" + Environment.NewLine +
				"　[Alt]キー ＋ a b c d e f g … セグメントを個別に点灯" + Environment.NewLine +
				"　[Space]キー … 消去（消灯）" + Environment.NewLine +
				"　[Esc]キー … 終了");
			
			ConsoleKeyInfo keyInfo;
			byte segPtn;
			char cc;
			bool dpOn = false;
			while ((keyInfo = Console.ReadKey(true)).Key != ConsoleKey.Escape)
			{
				if (keyInfo.Modifiers == ConsoleModifiers.Alt)
				{
					if (ConsoleKey.A <= keyInfo.Key && keyInfo.Key <= ConsoleKey.G)
					{
						segPtn = Segment.SegAtoG[keyInfo.Key - ConsoleKey.A];
						segled.PutSegPtn(segPtn);
					}
				}
				else
				{
					cc = keyInfo.KeyChar;
					if ('0' <= cc && cc <= '9')
						segled.PutNum(cc - '0');
					else if (('a' <= cc && cc <= 'z') || ('A' <= cc && cc <= 'Z'))
						segled.PutChar(cc);
					else if (cc == ' ')
						segled.PutSegPtn(Segment.SegNull);
					else if (cc == '.')
						segled.PutSegDP(dpOn = !dpOn);
					
					if (cc != '.') dpOn = false;
				}
			}
			Console.WriteLine();
			
			AppClose();
		}
		
		//アプリ終了
		private static void AppClose() { AppClose(null, ExitCode.NormalTermination); }
		private static void AppClose(string mes, ExitCode code)
		{
			//終了メッセージを表示する
			if (mes != null) Console.WriteLine(mes + Environment.NewLine);
			
			//FT232 クローズ
			if (ftdev != null && ftdev.IsOpen)
			{
				segled.BufferClear();
				segled.SetPortBits(0x00);
				ftdev.Close();
				Console.WriteLine("FT232 close");
			}
			
			Console.WriteLine("Press any key.");
			Console.ReadKey();
			Environment.Exit((int)code);
		}
	}
}
